;+
;
; PROCEDURE: get_tsy_params
;
; PURPOSE: this procedure will interpolate inputs, generate
;          tsyganenko model parameters and store them in a tplot 
;          variable that can be passed directly to the model 
;          procedure
;
;          relevant variables can be loaded from
;          ace data using the procedures:
;          load_ace_swepam.pro
;          load_ace_mag.pro
;          from ssl_general/key_param
; 
;
; KEYWORDS: dst_vec: an L length array storing the dst index
;
;           dst_time: an L length array storing the time values for
;           the dst index
;           
;           imf_tvar: tplot variable name storing the interplanetary 
;           magnetic field bvector in gsm
;
;           Np_tvar: tplot variable name storing the solar wind
;                   density(rho) cm^-3
;           
;           Vp_tvar: tplot variable name storing the proton velocity
;
;           model: a string, should be 'T96','T01' or 'TS04' (upper or
;           lower case)
;
;           newname(optional): the name of the output tplot variable
;           (default: t96_par','t01_par' or 'ts04_par' depending on 
;            selected model)
;
;           trange(optional): the time range over which the parameters
;           should range, if not set, this program will check the
;           timespan variable or prompt the user for a range
; 
;
pro get_tsy_params,dst_vec,dst_times,imf_tvar,Np_tvar,Vp_tvar,model,newname=newname

COMPILE_OPT idl2

if not keyword_set(trange) then tlims = timerange(/current) else tlims=trange

n = fix(tlims[1]-tlims[0],type=3)/300

ntimes=dindgen(n)*300+tlims[0]

if(n_elements(dst_vec) eq 1) then $
   dst = replicate(dst_vec,n_elements(ntimes)) $
else $
   dst = interpol(dst_vec,dst_times,ntimes)

if size(imf_tvar,/type) ne 7 then message,'imf_tvar must be a string'

if tnames(imf_tvar) eq '' then message,'imf_tvar must be a valid tplot variable'

get_data,imf_tvar,data=d

imf_times = d.x
imf_vec = d.y

if size(Np_tvar,/type) ne 7 then message,'Np_tvar must be a string'

if tnames(Np_tvar) eq '' then message,'Np_tvar must be a valid tplot variable'

get_data,Np_tvar,data=d

wind_den_times = d.x
wind_den = d.y

if size(Vp_tvar,/type) ne 7 then message,'Vp_tvar must be a string'

if tnames(Vp_tvar) eq '' then message,'Vp_tvar must be a valid tplot variable'

get_data,Vp_tvar,data=d

wind_spd_times = d.x
wind_spd = sqrt(total(d.y^2,2))

imf_y = interpol(imf_vec[*,1],imf_times,ntimes)
imf_z = interpol(imf_vec[*,2],imf_times,ntimes)
den = interpol(wind_den,wind_den_times,ntimes)
spd = interpol(wind_spd,wind_spd_times,ntimes)

pram = 1.667e-6*den*spd^2 

if strlowcase(model) eq 't01' then begin 
   geopack_getg,spd,imf_y,imf_z,out 

   par = {x:ntimes,y:[[pram],[dst],[imf_y],[imf_z],[out[*,0]],[out[*,1]],[dblarr(n)],[dblarr(n)],[dblarr(n)],[dblarr(n)]]}

   if not keyword_set(newname) then newname = 't01_par'

endif else if strlowcase(model) eq 'ts04' then begin 
   geopack_getw,den,spd,imf_z,out 

   par = {x:ntimes,y:[[pram],[dst],[imf_y],[imf_z],[out[*,0]],[out[*,1]],[out[*,2]],[out[*,3]],[out[*,4]],[out[*,5]]]}

   if not keyword_set(newname) then newname = 'ts04_par'

endif else if strlowcase(model) eq 't96' then begin

   par = {x:ntimes,y:[[pram],[dst],[imf_y],[imf_z],[dblarr(n)],[dblarr(n)],[dblarr(n)],[dblarr(n)],[dblarr(n)],[dblarr(n)]]}

   if not keyword_set(newname) then newname = 't96_par'

endif else message,'model name unrecognized'

store_data,newname,data=par

end
