;+
;NAME:
; thm_data2load
;PURPOSE:
; returns a list of variables that can be loaded for a given THEMIS
; instrument. For the most part, it calls the appropriate thm_load
; routine with the /valid_names keyword set. The dataypes and
; valid_names keywords are used inconsistently in the thm_loads, and
; do not allow for the distinction between level2 data that is
; to be input from level2 files, and level2 data the is to be input
; from level1 files and calbrated, cotrans'ed, etc... but has the same
; name as an L2 variable. Designed to be called from
; thm_ui_valid_dtype.pro
;CALLING SEQUENCE:
; dtyp = thm_data2load(instrument, level)
;INPUT:
; instrument = the THEMIS instrument: one of:
;           ['asi', 'ask', 'esa', 'efi', 'fbk', 'fft', 'fgm', $
;            'fit', 'gmag', 'mom', 'scm', 'sst', 'state', 'bau', 'hsk', 'trg']
; level = 'l1' for any data that can be gotten from the l1 file --
;         including calibrated, etc... 'l2' for data gotten from L2
;         files. 'l10' for data that only is loaded from L1 files. For
;         ESA data, 'L10' data and 'L1' data are gotten from the
;         packet files.
;OUTPUT:
; dtyp = a string array that can be used as an input to the datatype
;        keyword for the given instrument
;HISTORY:
; started on 31-Jan-2008, jmm, jimm@ssl.berkeley.edu, this is under
; development for the next 6 months or so.
;$LastChangedBy $
;$LastChangedDate  $
;$LastChangedRevision $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/trunk/idl/themis/common/thm_data2load.pro $
;-
Function thm_data2load, instrument, level

  instr = strcompress(strlowcase(instrument), /remove_all)
  lvl = strcompress(strlowcase(level), /remove_all)

  instr0 = ['asi', 'ask', 'esa', 'efi', 'fbk', 'fft', 'fgm', $
            'fit', 'gmag', 'mom', 'scm', 'sst', 'state', 'bau', 'hsk', 'trg']
  lvl0 = ['l1', 'l2', 'l10']

  xx = where(instr0 Eq instr)
  If(xx[0] Eq -1) Then Begin
    message, /info, 'Invalid Input: '+instrument
    print, 'Try, doc_library, ''thm_data2load'''
    return, ''
  Endif
  xx = where(lvl0 Eq lvl)
  If(xx[0] Eq -1) Then Begin
    message, /info, 'Invalid Input: '+level
    print, 'Try, doc_library, ''thm_data2load'''
    return, ''
  Endif

  load_routine = 'thm_load_'+instr
  resolve_routine, load_routine, /no_recompile
;special cases first, eventually all will be special cases to
;distinguish between l1, and l10
  Case instr Of
    'esa': Begin
      If(lvl Eq 'l1' Or lvl Eq 'l10') Then Begin
        thm_load_esa_pkt, /valid_names, datatype = vvv
      Endif Else Begin
        thm_load_esa, /valid_names, level = 'l2', datatype = vvv
      Endelse
    End
    'sst': Begin
      If(lvl Eq 'l1' Or lvl Eq 'l10') Then Begin
        ii = ['psif', 'psef', 'psir', 'pser']
        vv = ['ang', 'en', 'tot']
        vvv = ''
        For k = 0, n_elements(ii)-1 Do vvv = [vvv, ii[k]+'_'+vv]
        vvv = [vvv[1:3], 'psif_cnfg', 'psif_nspn', vvv[4:*]]
      Endif Else Begin
        vvv = ['psif', 'psef', 'psir', 'pser']+'_en_eflux'
      Endelse
    End
    'mom': Begin
      ii = ['peim', 'peem', 'psim', 'psem', 'ptim', 'ptem']
      vvv = ''
      If(lvl Eq 'l10') Then Begin ;for 'l10' set the raw keyword
        vv = ['density', 'flux', 'mftens', 'eflux']+'_raw'
        For k = 0, n_elements(ii)-1 Do vvv = [vvv, ii[k]+'_'+vv]
        vvv = [vvv[1:*], 'pxxm_pot_raw', 'pxxm_qf', 'pxxm_shft']
      Endif Else If(lvl Eq 'l1') Then Begin
        vv = ['density', 'flux', 'mftens', 'eflux', 'velocity', $
        'ptens', 'ptot', 'velocity_mag', 'ptens_mag', 't3_mag', 'mag']
        For k = 0, n_elements(ii)-1 Do vvv = [vvv, ii[k]+'_'+vv]
        vvv = [vvv[1:*], 'pxxm_pot', 'pxxm_qf', 'pxxm_shft']
      Endif Else Begin
        vv = ['density', 'flux', 'mftens', 'eflux', 'velocity_dsl', $
        'ptens', 'ptot', 'velocity_mag', 'ptens_mag', 't3_mag', $
        'mag', 'velocity_gse', 'velocity_gsm']
        For k = 0, n_elements(ii)-1 Do vvv = [vvv, ii[k]+'_'+vv]
        vvv = [vvv[1:*], 'pxxm_pot']
      Endelse
    End
    Else: Begin
      If(lvl Eq 'l1' Or lvl Eq 'l0') Then Begin
        call_procedure, load_routine, lvl = 'l1', /valid_names, datatype = vvv
      Endif Else Begin
        call_procedure, load_routine, lvl = 'l2', /valid_names, datatype = vvv
      Endelse
    End
  Endcase
  Return, vvv
End

        

      
 
