;+
;PROCEDURE: thm_crib_part_getspec
;
;  ***** A copy and paste crib *****
;
;PURPOSE:
;	A crib showing how to create energy and angular spectrograms using
;   THM_PART_GETSPEC.
;
;DETAILS
;   THM_PART_GETSPEC is a wrapper that takes user input and creates tplot
;   variables containing energy and/or angular spectra of ESA and SST data. The
;   tplot variables created end in 'en_eflux' for energy spectra and 'an_eflux'
;   plus a suffix corresponding to the type of angular spectrum specified by
;   the ANGLE keyword for angular spectra by default. The user can select a
;   range of probe(s), PROBE, and time range, TRANGE, of interest. The energy
;   range of interest, ERANGE, is specified in eV. The user can also choose to
;   create tplot variable for one or more data types, DATA_TYPE. The phi, PHI,
;   and theta, THETA, ranges of interest, specified in DSL coordinates, can also
;   be input, by the user. SUFFIX specifies a string that will be added to the
;   *an_eflux*' tplot variables.
;
;	If the phi range is greater than 360 degrees, then the phi bins at the
;   beginning of the phi range are added and wrapped around the end of phi
;   range. For example, if phi=[0,420], the phi bins corresponding to 0-60
;   degrees are appended to the top of the plot.
;
;	The START_ANGLE keyword specifies the start of the y-axis (phi). This is
;	useful to center the spectra plot on the y-axis at a particular phi angle.
;	If this	keyword is not set, the y-axis starts at the first angle input to
;	the PHI	keyword.
;
;   Use the ENERGY and ANGLE keywords to specify whether to create tplot
;   variables for energy and/or angular spectra. The ANGLE keyword is also used
;   to specify the type of angular spectrum created (e.g. phi, theta, etc).
;   Currently, 'phi' and 'theta' are the only valid angular spectrum types. If
;   neither of ENERGY and ANGLE keywords are specified, then then both are
;   turned on and the angular spectrum type defaults to phi.
;
;	When set, the AUTOPLOT keyword enables to the tplot variables to be
;	automatically plotted using some simple code located at the end of
;	THM_PART_GETSPEC. The tplot variables created are properly formatted by
;   THM_PART_MOMENTS2 and THM_PART_GETSPEC whether or not AUTOPLOT is set.
;   Setting AUTOPLOT will also create a default tplot title containing the
;   theta, phi, and energy ranges used to create the plot.
;
;   The NOMASK keyword is used to turn off the default sun pulse masking of SST
;   angle bins.
;
;   After reading in the user's input, THM_PART_GETSPEC then calls THM_LOAD_SST
;	and/or THM_LOAD_ESA_PKT to load the particle data.  THM_PART_MOMENTS2 is
;	then called to format the spectra data based on the user's input and create
;	the tplot variables.  THM_PART_MOMENTS2 calls THM_PART_GETANBINS in order to
;	determine which energy/angle bins should be turned on/off. If there's a mode
;	change, THM_PART_MOMENTS2 will re-call THM_PART_GETANBINS to account for any
;	changes in the mode's anglemap.
;
;NOTES:
;	- All angles are in DSL coordinates
;	- All datatypes except reduced are valid
;   - Theta input must satisfy: -90 < theta[0] < theta[1] < 90
;   - Phi input must be specified in ascending order in degrees
;     (e.g. [270, 450] or [-90, 90] to specify the 'daylight' hemisphere in DSL
;     coordinates)
;   - Incorporates improvements contained in v0.8.1
;
;CREATED BY:  Bryan Kerr
;VERSION: 0.8.11
;  $LastChangedBy: bckerr $
;  $LastChangedDate: 2008-02-20 15:29:55 -0800 (Wed, 20 Feb 2008) $
;  $LastChangedRevision: 2389 $
;  $URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/trunk/idl/themis/examples/thm_crib_part_getspec.pro $
;-

; thm_crib_part_getspec_01
; Example of plotting the angular phi spectrum in phi for 2 days of data
; beginning 12:30pm, June 17, 2007 for probes D and E, for peif and peef data
; for energies between 15 and 40 keV, for all thetas, and for all phi angles.
; Setting the start_angle to -35 degrees sets up tplot variable so that the phi
; angles are plotted from -35 to 325 degrees.

del_data, '*'
thm_part_getspec, probe=['d','e'], trange=['07-06-17/12:30','07-06-19/12:30'], $
                  theta=[-90,90], phi=[0,360], erange=[15000,40000], $
                  data_type=['peif','peef'], start_angle=-35, /autoplot, $
                  angle='phi'




; thm_crib_part_getspec_02
; Example of plotting the angular phi spectrum in phi for 2 days of data beginning
; 12:30pm, June 17, 2007 for probes A and B, for the psif and psef data types
; for all energies between 100 and 1000 eV, a theta angle between -45 and +45
; degrees, and for all phi angles with an additional 90 degrees wrapped to the
; top of the plot.

del_data, '*'
thm_part_getspec, probe=['a', 'b'], trange=['07-06-17/12:30','07-06-19/12:30'], $
                  theta=[-45,45], phi=[0,450], erange=[5e4, 1e6], $
                  data_type=['psif','psef'], /autoplot, angle='phi'




; thm_crib_part_getspec_03
; Example of plotting the energy spectrum for 1.5 hours of data beginning
; 10:30am, March 23, 2007 for probes A and E, for the peif, psif, peef, and psef
; data types for all energies for all theta angles, and for dayside phi angles.

; Alternatively, the 'dayside' angles can also be specified as phi=[270,450].


del_data, '*'
thm_part_getspec, probe=['a','e'], trange=['07-03-23/10:30','07-03-23/12'], $
                  theta=[-90,90], phi=[-90,90], $
                  data_type=['peif', 'psif', 'peef', 'psef'], $
                  /energy, /autoplot




; thm_crib_part_getspec_04
; Example of plotting the angular phi spectrum for 1.5 hours of data beginning
; 10:30am, March 23, 2007 for probes A and E, for the peib, psif, peeb, and psef
; data types for the energy range between 10 and 100 keV, for all theta angles,
; and for all phi angles. A start_angle for phi of 180 degrees is specified to
; center the plot at phi = 360. A suffix, '_march23', is added to all tplot
; variables.  The NOMASK keyword is used to deactivate the default sunpulse mask
; for SST data.

; This example also shows how to create your own plot without using the AUTOPLOT
; keyword.

del_data, '*'
thm_part_getspec, probe=['a','e'], trange=['07-03-23/10:30','07-03-23/12'], $
                  theta=[-90,90], phi=[0,360], start_angle = 180, $
                  data_type=['peib', 'psif', 'peeb', 'psef'], $
                  erange=[10000, 100000], suffix='_march23', angle='phi', /nomask

tplot,'th*an_eflux_phi_march23'




; thm_crib_part_getspec_05
; Example plotting full energy and phi angular spectra for full, reduced, and
; burst iESA data
del_data, '*'
thm_part_getspec, probe=['e'], trange=['07-03-23/11:13','07-03-23/12'], $
                  theta=[-90,90], phi=[0,360], start_angle=180, $
                  data_type=['peif','peib','peir'], erange=[0, 100000], $
                  /energy, angle='phi', /autoplot




; thm_crib_part_getspec_06
; Example plotting full energy and phi angular spectra for SST and ESA
; reduced data.
del_data, '*'
thm_part_getspec, probe=['e'], trange=['07-06-17/9:50','07-06-17/10:50'], $
                  theta=[-90,90], phi=[0,360], $
                  data_type=['psir','pser','peir','peer'], $
                  /energy, /autoplot, angle='phi', /nomask




; thm_crib_part_getspec_07
; Example plotting full eESA theta angular spectra for all 5 probes
del_data, '*'
thm_part_getspec, probe=['a','b','c','d','e'], trange=['07-03-23/11:13','07-03-23/12'], $
                  theta=[-90,90], phi=[0,360], data_type=['peif'], /autoplot, $
                  angle='theta'




; thm_crib_part_getspec_08
; Example plotting full iESA pitch angular spectra for all 5 probes
del_data, '*'
thm_part_getspec, probe=['a','b','c','d','e'], trange=['07-03-23/11:15','07-03-23/12'], $
                  data_type=['peif'], angle='pa', /autoplot




; thm_crib_part_getspec_09
; Create a tplot variable, thc_pser_an_eflux_phi,containing the phi spectrum
; for 6-angle pser data for probe c.
del_data, '*'
thm_part_getspec, probe=['c'], trange=['08-02-12/03:37','08-02-12/03:45'], $
                  data_type='pser', angle='phi', /nomask