;+
;PROCEDURE: thm_part_moments2
;PURPOSE: Calculates moments and spectra for themis particle distributions.
;
;SEE ALSO:
;	THM_CRIB_PART_GETSPEC, THM_PART_GETSPEC, THM_PART_GETANBINS, THM_LOAD_SST,
;   THM_LOAD_ESA_PKT
;
;MODIFIED BY:	Bryan kerr from Davin Larson's thm_part_moments
;HISTORY:
; v0.3  12/04/07: Improved ability to better handle phi input from
;                 THM_PART_GETSPEC.
; v0.4  12/13/07: Added doangle, doenergy, phiwrap keywords, moved p_start_angle
;                 and p_end_angle code from thm_part_getspec to here. Added
;                 test to check whether all energy/angle bins are turned off.
; v0.5  01/04/08: Changed tplotsuffix to suffix. Reduced banding of masked SST
;                 data. DOANGLE keyword now specifies type of angular spectrum
;                 and adds a corresponding suffix to an_eflux tplot variable
;                 names. Now reduces en_eflux tplot variable to a single time
;                 series when only one energy bin is requested. Fixed NAN bug
;                 that corrupted SST energy bins that were turned off.
; v0.6  01/09/08: Added reduced mode (peir) capability.
; v0.6.3 01/15/08: All reduced modes implemented. Energy/angle maps now passed
;                  to THM_PART_GETANBINS. *en_eflux tplot variables resized to
;                  actual number of energies requested. Fixed bug that
;                  prevented plotting of single-angle angular spectrograms.
; v0.7  01/31/08: Added theta spectrograms capability. Angular spectra code now
;                 generalized to calculate dOmega for arbitrary angle maps.
;                 Fixed bug that didn't allow plotting of spectra for a single
;                 angle bin.
; v0.8  02/04/08: Alpha release of pitch angle spectragram capability.
; v0.8.1 02/20/08: Fixed bug incorrectly weighting angle bins introduced in v0.7.
;                  Fixed bug that didn't account for negative minphi angles.
;                  Modified code to better handle 6-angle pser data.
; Things to do:   Fix bug that causes the code to bomb when a phi range is
;                 specified for theta spectragrams.
;
;VERSION: 0.8.1
;  $LastChangedBy: bckerr $
;  $LastChangedDate: 2008-02-20 11:04:07 -0800 (Wed, 20 Feb 2008) $
;  $LastChangedRevision: 2386 $
;  $URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/trunk/idl/themis/spacecraft/particles/moments/thm_part_moments2.pro $
;-


pro thm_part_moments2, instruments_types=instruments, probes=probes,  $  ;moments=moms,  $
                       moments_types=moments_types, $
                       verbose=verbose, $
                       trange=trange, $
                       tplotnames=tplotnames, $
                       tplotsuffix=tplotsuffix, $
                       set_opts=set_opts, $
                       scpot_suffix=scpot_suffix, mag_suffix=mag_suffix, $ ;'inputs: suffix specifying source of magdata and scpot (name - 'th?')
                       comps=comps, get_moments=get_moments,usage=usage, $
                       units=units, theta=theta, phi=phi, pitch=pitch, $
                       erange=erange, start_angle=start_angle, doangle=doangle, $
                       doenergy=doenergy, wrapphi=wrapphi, _extra=ex

defprobes = '?'
definstruments = 'p??f'
defmoments='density velocity t3 magt3'
start = systime(1)

vprobes      = ['a','b','c','d','e']
if n_elements(probes) eq 1 then if probes eq 'f' then vprobes=['f']
vinstruments = ['peif','peef','psif','psef','peir','peer','psir','pser','peib','peeb']
vmoments     = strlowcase(strfilter(tag_names(moments_3d()),['TIME','ERANGE','MASS','VALID'],/negate))

if keyword_set(comps) or keyword_set(get_moments)  then begin
   scp = scope_traceback(/struct)
   nscp = n_elements(scp)
   dprint,'Keyword names have changed. Please change the calling routine: ',scp[nscp-2].routine
   usage=1
endif
if keyword_set(usage) then begin
   scp = scope_traceback(/struct)
   nscp = n_elements(scp)
   dprint,'Typical usage: '  ,scp[nscp-1].routine  ,", INSTRUMENT='pe?f', PROBES='a', MOMENTS='density velocity'"
   dprint,'Valid inputs:'
   dprint,'  PROBES=',"'"+vprobes+"'"
   dprint,'  INSTRUMENTS=',"'"+vinstruments+"'"
   dprint,'  MOMENTS=',"'"+vmoments+"'"
   return
endif

;probea = thm_check_valid_name(size(/type,probes) eq 7 ? probes : '*',vprobes)

probes_a       = strfilter(vprobes,size(/type,probes) eq 7 ? probes : defprobes,/fold_case,delimiter=' ',count=nprobes)
instruments_a  = strfilter(vinstruments,size(/type,instruments)  eq 7 ? instruments  : definstruments,/fold_case,delimiter=' ',count=ninstruments)
moments_a      = strfilter(vmoments,  size(/type,moments_types)  eq 7 ? moments_types  : defmoments  ,/fold_case,delimiter=' ',count=nmoments)
tplotnames=''
if not keyword_set(tplotsuffix) then tplotsuffix=''

;if keyword_set(get_moments) then if not keyword_set(comps)  then comps = ['density','velocity','t3']
;comps = strfilter(vmoments,compmatch,/fold_case,delimiter=' ')
dprint,dlevel=2,/phelp,probes_a
dprint,dlevel=2,/phelp,instruments_a
dprint,dlevel=2,/phelp,moments_a

if size(/type,units) ne 7 then units='eflux'

phi1 = phi[0]
phi2 = phi[1]
if keyword_set(start_angle) then begin
   p_start_angle = start_angle
   p_end_angle = start_angle + phi2 + wrapphi - phi1
endif else begin
   start_angle = phi1
   p_start_angle = phi1
   p_end_angle = phi2 + wrapphi
endelse
theta1 = theta[0]
theta2 = theta[1]
th_start_angle = theta1
th_end_angle = theta2
pitch1 = pitch[0]
pitch2 = pitch[1]
pa_start_angle = pitch1
pa_end_angle = pitch2

if ~ keyword_set(doangle) then doangle='none'

;-----------------------------------------------------------------------

for p = 0,nprobes-1 do begin
    probe= probes_a[p]
    thx = 'th'+probe
    for t=0,ninstruments-1 do begin
        instrument = instruments_a(t)
        format = thx+'_'+instrument
        times= thm_part_dist(format,/times)
;        ns = n_elements(times) * keyword_set(times)
        if keyword_set(trange) then  tr = minmax(time_double(trange)) else tr=[1,1e20]
        ind = where(times ge tr[0] and times le tr[1],ns)

        if ns eq 0 then dprint, 'WARNING: No data for ', format else dprint,format,ns,' elements'

        if ns gt 0  then begin
           if keyword_set(mag_suffix) then begin
                dprint,dlevel=2,verbose=verbose,'Interpolating mag data from: ',thx+mag_suffix
                magf = data_cut(thx+mag_suffix,times[ind])
           endif
           if keyword_set(scpot_suffix) then begin
                dprint,dlevel=2,verbose=verbose,'Interpolating sc potential from:',thx+scpot_suffix
                scpot = data_cut(thx+scpot_suffix,times[ind])
           endif
           dat = thm_part_dist(format,index=0,_extra=ex)

           ;get angle bin arrays
           mode = dat.mode
           nrg = average(dat.energy,2)
           avgphi=average(dat.phi,1)
           avgtheta=average(dat.theta,1)
           thm_part_getanbins, theta=theta, phi=phi, pitch=pitch, erange=erange, $
                               data_type=instrument, en_an_bins=en_an_bins, $
                               an_bins=an_bins, en_bins=en_bins, nrg=nrg, $
                               avgphi=avgphi, avgtheta=avgtheta
           if total(an_bins) eq 0 then begin
              print, ''
              dprint, 'WARNING: No angle bins turned on for ',instrument, $
                      ' data. Moving on to next data_type.'
              continue
           endif
           if total(en_an_bins) eq 0 then begin
              print, ''
              dprint, 'WARNING: No energy bins turned on for ',instrument, $
                      ' data. Moving on to next data_type.'
              continue
           endif

           if keyword_set(nmoments) then moms = replicate( moments_3d(), ns )
           time = replicate(!values.d_nan,ns)
           maxnrgs = strmid(instrument,1,1) eq 'e' ? 32 : 16
           dprint,dlevel=3,/phelp,maxnrgs
           spec = replicate(!values.f_nan, ns, maxnrgs )
           energy =  replicate(!values.f_nan,ns, maxnrgs )
           phispec = replicate(!values.f_nan,ns, 28) ; might have to change the 2nd dim max
           thetaspec = replicate(!values.f_nan,ns, 28) ; might have to change the 2nd dim max
           paspec = replicate(!values.f_nan,ns,n_elements(avgphi))
           pa_red = replicate(!values.f_nan,ns,n_elements(avgphi))
           last_angarray = 0
           for i=0L,ns-1  do begin
              dat = thm_part_dist(format,index=ind[i],_extra=ex)
              mode = dat.mode
              avgphi=average(dat.phi,1)
              avgtheta=average(dat.theta,1)
              nrg = average(dat.energy,2)
              if i eq 0 then pa_red = replicate(!values.f_nan,ns,n_elements(avgphi))
              if i eq 0 then paspec = replicate(!values.f_nan,ns,n_elements(avgphi))
              angarr=[dat.theta,dat.phi,dat.dtheta,dat.dphi]
              if array_equal(last_angarray,angarr) eq 0 then begin   ; Sense change in mode
                 last_angarray = angarr
                 domega = 0
                 dprint,verbose=verbose,'Index=',i,' New mode at: ',time_string(dat.time), $
                                        '.  Mode: ', mode
                 ; renew angle bin maps
                 thm_part_getanbins, theta=theta, phi=phi, pitch=pitch, erange=erange, $
                                    data_type=instrument, en_an_bins=en_an_bins, $
                                    an_bins=an_bins, en_bins=en_bins, nrg=nrg, $
                                    avgphi=avgphi, avgtheta=avgtheta
              endif
              time[i] = dat.time
              dim = size(/dimen,dat.data)
              dim_spec = size(/dimensions,spec)

              if keyword_set(units) then begin
                 udat = conv_units(dat,units+'')
                 bins = udat.bins
                 dim_data = size(/dimensions,udat.data)
                 nd = size(/n_dimensions,udat.data)
                 dim_bins = size(/dimensions,bins)
                 if array_equal(dim_data,dim_bins) eq 0 then bins = replicate(1,udat.nenergy) # bins   ; special case for McFadden's 3D structures
                 ; Program caused arithmetic error: Floating illegal operand
                 ; whenever there's a zero in total(bins ne 0,2)
                 bins = bins * en_an_bins
                 sp = nd eq 1 ? udat.data * bins  : total(udat.data * (bins gt 0),2) / total(bins gt 0,2)
                 en = nd eq 1 ? udat.energy * bins : total(udat.energy * (bins gt 0),2) / total(bins gt 0,2)
                 if nd eq 1 then begin
                    sp = (udat.data * (bins gt 0)) / (bins gt 0)
                    en = (udat.energy * (bins gt 0)) / (bins gt 0)
                 endif
                 spec[ i, 0:dim[0]-1] = sp
                 energy[ i,0:dim[0]-1] = en


                 ; compute phi angle spectra
                 if doangle eq 'phi' then begin

                 asp = nd eq 1 ? udat.data : total(udat.data * (bins ne 0),1) / total(bins ne 0,1) ; avg over all nrg bins
                 finite_ind = where(finite(asp),COMPLEMENT=infinite_ind, NCOMPLEMENT=ninfinite)
                 if ninfinite gt 0 then begin
                    asp[infinite_ind] = 0
                 endif
                 bad_ang = where(finite(total(udat.bins,1),/NAN),n); find sst "sunmasked" bins
                 if n gt 0 then asp[bad_ang]=!values.f_nan ; re-insert NaNs for sunmasked" bins

;                 ;test section
;                 asp = fltarr(88) + 1e5 ;test remove when done
;                 asp = fltarr(64) + 1e5 ;test remove when done
                 ;asp =fltarr(6) + 1e5 ;test remove when done
                 ;asp[3:4] = 1e7
                 ;asp[2:5] = 0
;                 ;aspi1=[0,4,8,9,16,17];,24,40,56,72]
;                 aspi1=[0,1,2,3,4,5,6,7]
;                 aspi2=[4,5,6,7]
;                 asp(0:3) = 1e4
;                 asp(16:19) = 1e6
;                 aspi3=[1,2,3,4,10,11,18,19,5,6,7,8,30,62,78,46,31,63,77,47,28,60,76,44]
;                 asp[aspi3] = 0

                 en_binsi = where(en_bins gt 0, n_en_binsi)
                 dphi = average(udat.dphi[en_binsi,*],1)
                 dtheta = average(udat.dtheta[en_binsi,*],1)
                 asp_on = intarr(n_elements(asp))
                 asp_on_ind = where(asp gt 0,n_asp_on_ind) ; index of angle bin where spectra has been calc'd
                 if n_asp_on_ind gt 0 then begin
                    an_bins_ind = where(an_bins eq 1)
                    asp_on(asp_on_ind) = 1
;                    aphi = average(udat.phi[en_binsi,*],1) ; avg phi of nrgies collected in each angle bin
                    aphi = average(udat.phi,1) ; avg phi of all available nrgies in each angle bin
;                    maxphi = max(udat.phi[en_binsi,*],dimension=1, min=minphi) ; min/max phi of energies collected in each angle bin
                    ; use this if you want the min/max phi of each bin regardless of nrgies picked
                    maxphi = max(udat.phi,dimension=1, min=minphi)
                    if strmid(instrument,1,1) eq 's' then begin
                       maxphi = aphi + 11 ; based on SST angular response FWHM of ~22 degrees
                       minphi = aphi - 11 ; based on SST angular response FWHM of ~22 degrees
                    endif else begin
                       if array_equal(maxphi, aphi) then begin
                          maxphi = aphi + dphi/4 ;the 4 is arbitrary since angular response not in dist.
                          minphi = aphi - dphi/4 ;the 4 is arbitrary since angular response not in dist.
                       endif
                    endelse

                    ; convert any phi's gt 360
                    gt360_ind = where(aphi gt 360,gt360_count)
                    if gt360_count ne 0 then begin
                       aphi(gt360_ind) = aphi(gt360_ind) - 360
                       maxphi(gt360_ind) = maxphi(gt360_ind) - 360
                       minphi(gt360_ind) = minphi(gt360_ind) - 360
                    endif

;                    gt360i = where(minphi gt 360, gt360c)
;                    if gt360c ne 0 then minphi(gt360i) = minphi(gt360i) - 360

;                    gt360i = where(maxphi gt 360, gt360c)
;                    if gt360c ne 0 then maxphi(gt360i) = maxphi(gt360i) - 360

;                    lt360i = where(minphi lt 360, lt360c)
;                    if lt360c ne 0 then minphi(lt360i) = minphi(lt360i) + 360

;                    lt360i = where(maxphi lt 360, lt360c)
;                    if lt360c ne 0 then maxphi(lt360i) = maxphi(lt360i) + 360

                    atheta = average(udat.theta[en_binsi,*],1) ; avg theta of energies collected in each angle bin
                    aphi_on = aphi(an_bins_ind) ; array of aphi reduced to only bins that are on
                    maxphi_on = maxphi(an_bins_ind)
                    minphi_on = minphi(an_bins_ind)
                    atheta_on = atheta(an_bins_ind) ; array of atheta reduced to only bins that are on
                    dphi_on = average(udat.dphi(0,an_bins_ind),1)
                    dtheta_on = average(udat.dtheta(0,an_bins_ind),1)
                    ; might have to test the fact that SORT might treat identical
                    ; elements differently on different OS's won't affect output
                    phi_dat_ind = uniq(aphi_on, sort(aphi_on)); udat.phi bin #'s of unique and sorted phi's
                    aphi_red = aphi_on(phi_dat_ind) ; sort and pick out unique avgphi's
                    maxphi_red = maxphi_on(phi_dat_ind)
                    minphi_red = minphi_on(phi_dat_ind)
                    atheta_red = atheta_on(phi_dat_ind) ; sort and pick out unique avgtheta's
                    dphi_red = dphi_on(phi_dat_ind)
                    nphi = n_elements(aphi_red)

                    for a = 0L,nphi-1 do begin
                       ; look for bins that fall in current bin's primary angle range
                       if minphi_red[a] lt 0 then begin
                          combins = where((aphi*an_bins + dphi*an_bins/2) gt minphi_red[a] $
                                           AND (aphi*an_bins - dphi*an_bins/2) lt maxphi_red[a])
                          wrapbin = where((aphi*an_bins + dphi*an_bins/2) ge 360,nwrapbin) ;account for bins with boundary at 360 degrees
                          if nwrapbin gt 0 then begin
                             if (aphi[wrapbin]*an_bins[wrapbin] + $
                                dphi[wrapbin]*an_bins[wrapbin]/2) gt (minphi_red[a] + 360) then begin
                                combins = [combins, wrapbin]
                             endif
                          endif

                          large_dphi_ind = where((aphi[combins] + dphi[combins]/2) gt maxphi_red[a] $
                                              AND (aphi[combins] - dphi[combins]/2) lt minphi_red[a], $
                                              nlarge_dphi, COMPLEMENT=small_dphi_ind, $
                                              NCOMPLEMENT=nsmall_dphi)
                       endif else begin
                          combins = where((aphi*an_bins + dphi*an_bins/2) gt minphi_red[a] $
                                           AND (aphi*an_bins - dphi*an_bins/2) lt maxphi_red[a], ncombins)
                          wrapbin = where((aphi*an_bins + dphi*an_bins/2) ge 360,nwrapbin) ;account for bins with boundary at 360 degrees
                          fullbins = where(dphi*an_bins eq 360, nfullbins)
                          if ncombins eq 1 AND nfullbins gt 0 then begin
                             combins = [fullbins, combins]
                             large_dphi_ind = indgen(n_elements(combins))
                             nlarge_dphi = n_elements(large_dphi_ind)
                             nsmall_dphi = 0
                          endif else begin

                             large_dphi_ind = where((aphi[combins] + dphi[combins]/2) gt maxphi_red[a] $
                                              AND (aphi[combins] - dphi[combins]/2) lt minphi_red[a], $
                                              nlarge_dphi, COMPLEMENT=small_dphi_ind, $
                                              NCOMPLEMENT=nsmall_dphi)
                          endelse
                       endelse
;                       large_dphi_ind = where((aphi[combins] + dphi[combins]/2) gt maxphi_red[a] $
;                                           AND (aphi[combins] - dphi[combins]/2) lt minphi_red[a], $
;                                           nlarge_dphi, COMPLEMENT=small_dphi_ind, $
;                                           NCOMPLEMENT=nsmall_dphi)

                       if nsmall_dphi gt 0 then begin
                          phispec_b = fltarr(nsmall_dphi)
                          del_omega_rad_b = phispec_b

                          for b = 0,nsmall_dphi-1 do begin
                             temp_ind = combins[small_dphi_ind[b]]
                             if strmid(instrument,1,1) eq 's' then begin
                                ;hw = min(dphi_red) / 4 ; fix in absence of sst phi-collection range
                                hw = (maxphi_red[a] - minphi_red[a])/2
                             endif else begin
                                hw = (maxphi_red[a] - minphi_red[a] + $
                                     (maxphi_red[a] - minphi_red[a]) / (n_en_binsi - 1)) / 2
                             endelse
                             if aphi_red[a] gt aphi[temp_ind] then begin
                                w = (aphi[temp_ind] + dphi[temp_ind]/2) - (aphi_red[a] - hw)
                             endif else if aphi_red[a] lt aphi[temp_ind] then begin
                                if (aphi[temp_ind] + dphi[temp_ind]) gt aphi_red[a] $
                                   AND (aphi[temp_ind] - dphi[temp_ind]) gt aphi_red[a] then begin
                                   w = aphi_red[a] + hw + 360 - $
                                       (aphi[temp_ind] + dphi[temp_ind]/2)
                                endif else begin
                                   w = (aphi_red[a] + hw) - (aphi[temp_ind] - dphi[temp_ind]/2)
                                endelse
                             endif else begin
                                w = 2 * hw
                             endelse
                             th0_rad = (atheta[temp_ind] + dtheta[temp_ind]/2.)*!pi/180.
                             th1_rad = (atheta[temp_ind] - dtheta[temp_ind]/2.)*!pi/180.
                             w_rad = w * !pi/180.
                             del_omega_rad_b[b] = w_rad * (sin(th0_rad) - sin(th1_rad))
                             if ~ finite(asp[temp_ind]) then del_omega_rad_b[b] = 0.
                             phispec_b[b] = asp[temp_ind] * del_omega_rad_b[b]
                             ;phispec_b[b] = asp[temp_ind] * finite(asp[temp_ind]);testing line
                          endfor

                       endif else begin
                          phispec_b = 0.
                          del_omega_rad_b = 0.
                       endelse
                       phispec_c = fltarr(nlarge_dphi)
                       del_omega_rad_c = phispec_c

                       for c = 0,nlarge_dphi-1 do begin
                          temp_ind = combins[large_dphi_ind[c]]
                          th0_rad = (atheta[temp_ind] + dtheta[temp_ind]/2.)*!pi/180.
                          th1_rad = (atheta[temp_ind] - dtheta[temp_ind]/2.)*!pi/180.
                          if strmid(instrument,1,1) eq 's' then begin
                             w = maxphi_red[a] - minphi_red[a] ; fix in absence of sst phi-collection range
                          endif else begin
                             w = maxphi_red[a] - minphi_red[a] + $
                                 (maxphi_red[a] - minphi_red[a]) / (n_en_binsi - 1)
                          endelse
                          w_rad = w * !pi/180
                          del_omega_rad_c[c] = w_rad * (sin(th0_rad) - sin(th1_rad))
                          if ~ finite(asp[temp_ind]) then del_omega_rad_c[c] = 0.
                          phispec_c[c] = asp[temp_ind] * del_omega_rad_c[c]
                          ;phispec_c[c] = asp[temp_ind] * finite(asp[temp_ind]);testing line
                       endfor

                       del_omega_rad2 = total(del_omega_rad_b) + total(del_omega_rad_c)
                       phispec[i,a] = (total(phispec_b,/NAN) + total(phispec_c,/NAN)) / del_omega_rad2
                       ;phispec[i,a] = (total(phispec_b,/NAN) + total(phispec_c,/NAN));testing line
                    endfor ; loop over phi's

                 endif else begin
                    ;all bins have zero spectra data
                    if i gt 0 then begin
                       nphi_bins = n_elements(phispec[i-1])
                       phispec[i] = fltarr(nphi_bins)
                    endif else begin
                      phispec[i,*] = 0; the initial size of the phispec array
                      nphi = n_elements(phispec[i,*])
                      aphi_red = fltarr(nphi)
                      dphi_red = fltarr(nphi)
                    endelse
                 endelse
                 angspec = phispec
                 ; end compute phi angle spectra





                 endif else if doangle eq 'theta' then begin
                 ; compute theta angle spectra

                 asp = nd eq 1 ? udat.data : total(udat.data * (bins ne 0),1) / total(bins ne 0,1) ; avg over all nrg bins
                 finite_ind = where(finite(asp),COMPLEMENT=infinite_ind, NCOMPLEMENT=ninfinite)
                 if ninfinite gt 0 then begin
                    asp[infinite_ind] = 0
                 endif
                 bad_ang = where(finite(total(udat.bins,1),/NAN),n); find sst "sunmasked" bins
                 if n gt 0 then asp[bad_ang]=!values.f_nan ; re-insert NaNs for sunmasked" bins

;                 ;test section
;                 asp = fltarr(88) + 1e5 ;test remove when done
;                 asp = fltarr(64) + 1e5 ;test remove when done
                 ;asp =fltarr(6) + 1e5 ;test remove when done
                 ;asp[0] = 1e7
                 ;asp[3:4] = 0
;                 ;aspi1=[0,4,8,9,16,17];,24,40,56,72]
;                 aspi1=[0,1,2,3,4,5,6,7]
;                 aspi2=[4,5,6,7]
;                 asp(0:3) = 1e4
;                 asp(16:19) = 1e6
;                 aspi3=[1,2,3,4,10,11,18,19,5,6,7,8,30,62,78,46,31,63,77,47,28,60,76,44]
;                 asp[aspi3] = 0

                 en_binsi = where(en_bins gt 0, n_en_binsi)
                 dphi = average(udat.dphi[en_binsi,*],1)
                 dtheta = average(udat.dtheta[en_binsi,*],1)
                 asp_on = intarr(n_elements(asp))
                 asp_on_ind = where(asp gt 0,n_asp_on_ind) ; index of angle bin where spectra has been calc'd
                 if n_asp_on_ind gt 0 then begin
                    an_bins_ind = where(an_bins eq 1)
                    asp_on(asp_on_ind) = 1
;                    aphi = average(udat.phi[en_binsi,*],1) ; avg phi of nrgies collected in each angle bin
                    aphi = average(udat.phi,1) ; avg phi of all available nrgies in each angle bin
;                    maxphi = max(udat.phi[en_binsi,*],dimension=1, min=minphi) ; min/max phi of energies collected in each angle bin
                    ; use this if you want the min/max phi of each bin regardless of nrgies picked
                    maxphi = max(udat.phi,dimension=1, min=minphi)
                    maxphi = max(udat.phi,dimension=1, min=minphi)
                    if strmid(instrument,1,1) eq 's' then begin
                       maxphi = aphi + 11 ; based on SST angular response FWHM of ~22 degrees
                       minphi = aphi - 11 ; based on SST angular response FWHM of ~22 degrees
                    endif else begin
                       if array_equal(maxphi, aphi) then begin
                          maxphi = aphi + dphi/4 ;the 4 is arbitrary since angular response not in dist.
                          minphi = aphi - dphi/4 ;the 4 is arbitrary since angular response not in dist.
                       endif
                    endelse

;                    atheta = average(udat.theta[en_binsi,*],1) ; avg theta of energies collected in each angle bin
                    atheta = average(udat.theta,1) ; avg theta of energies collected in each angle bin
;                    maxtheta = max(udat.theta[en_binsi,*],dimension=1, min=mintheta) ; min/max phi of energies collected in each angle bin
                    ; use this if you want the min/max phi of each bin regardless of nrgies picked
                    maxtheta = max(udat.theta,dimension=1, min=mintheta)
                    if array_equal(maxtheta, atheta) then begin
                       maxtheta = atheta + dtheta/2
                       mintheta = atheta - dtheta/2
                    endif

                    ; convert any phi's gt 360
                    gt360_ind = where(aphi gt 360,gt360_count)
                    if gt360_count ne 0 then begin
                       aphi(gt360_ind) = aphi(gt360_ind) - 360
                       maxphi(gt360_ind) = maxphi(gt360_ind) - 360
                       minphi(gt360_ind) = minphi(gt360_ind) - 360
                    endif

                    aphi_on = aphi(an_bins_ind) ; array of aphi reduced to only bins that are on
                    maxphi_on = maxphi(an_bins_ind)
                    minphi_on = minphi(an_bins_ind)
                    atheta_on = atheta(an_bins_ind) ; array of atheta reduced to only bins that are on
                    maxtheta_on = maxtheta(an_bins_ind)
                    mintheta_on = mintheta(an_bins_ind)
                    dphi_on = average(udat.dphi(0,an_bins_ind),1)
                    dtheta_on = average(udat.dtheta(0,an_bins_ind),1)
                    ; might have to test the fact that SORT might treat identical
                    ; elements differently on different OS's won't affect output
                    phi_dat_ind = uniq(aphi_on, sort(aphi_on)); udat.phi bin #'s of unique and sorted phi's
                    theta_dat_ind = uniq(atheta_on, sort(atheta_on)); udat.theta bin #'s of unique and sorted theta's
                    aphi_red = aphi_on(phi_dat_ind) ; sort and pick out unique avgphi's
                    atheta_red = atheta_on(theta_dat_ind) ; sort and pick out unique avgtheta's
                    maxphi_red = maxphi_on(phi_dat_ind)
                    minphi_red = minphi_on(phi_dat_ind)
                    maxtheta_red = maxtheta_on(theta_dat_ind)
                    mintheta_red = mintheta_on(theta_dat_ind)
                    dphi_red = dphi_on(phi_dat_ind)
                    nphi = n_elements(aphi_red)
                    dtheta_red = dtheta_on(theta_dat_ind)
                    ntheta = n_elements(atheta_red)

                    for a = 0L,ntheta-1 do begin
                       ; look for bins that fall in current bin's primary angle range
                       combins = where((atheta*an_bins + dtheta*an_bins/2) gt mintheta_red[a] $
                                        AND (atheta*an_bins - dtheta*an_bins/2) lt maxtheta_red[a])
                       ncombins = n_elements(combins)
                       large_dtheta_ind = where((atheta[combins] + dtheta[combins]) gt maxtheta_red[a] $
                                              AND (atheta[combins] - dtheta[combins]) lt mintheta_red[a], $
                                              nlarge_dtheta, COMPLEMENT=small_dtheta_ind, $
                                              NCOMPLEMENT=nsmall_dtheta)
                       if nsmall_dtheta gt 0 then begin
                          thetaspec_b = fltarr(nsmall_dtheta)
                          del_omega_rad_b = thetaspec_b

                          for b = 0,nsmall_dtheta-1 do begin
                             temp_ind = combins[small_dtheta_ind[b]]
                             if strmid(instrument,1,1) eq 's' then begin
                                ;hw = min(dphi_red) / 4 ; fix in absence of sst phi-collection range
                                hw = (maxphi_red[a] - minphi_red[a])/2
                             endif else begin
                                hw = (maxphi_red[a] - minphi_red[a] + $
                                     (maxphi_red[a] - minphi_red[a]) / (n_en_binsi - 1)) / 2
                             endelse
                             if aphi_red[a] gt aphi[temp_ind] then begin
                                w = (aphi[temp_ind] + dphi[temp_ind]/2) - (aphi_red[a] - hw)
                             endif else if aphi_red[a] lt aphi[temp_ind] then begin
                                if (aphi[temp_ind] + dphi[temp_ind]) gt aphi_red[a] $
                                   AND (aphi[temp_ind] - dphi[temp_ind]) gt aphi_red[a] then begin
                                   w = aphi_red[a] + hw + 360 - $
                                       (aphi[temp_ind] + dphi[temp_ind]/2)
                                endif else begin
                                   w = (aphi_red[a] + hw) - (aphi[temp_ind] - dphi[temp_ind]/2)
                                endelse
                             endif else begin
                                w = 2 * hw
                             endelse
                             th0_rad = (atheta[temp_ind] + dtheta[temp_ind]/2.)*!pi/180.
                             th1_rad = (atheta[temp_ind] - dtheta[temp_ind]/2.)*!pi/180.
                             w_rad = w * !pi/180.
                             del_omega_rad_b[b] = w_rad * (sin(th0_rad) - sin(th1_rad))
                             if ~ finite(asp[temp_ind]) then del_omega_rad_b[b] = 0.
                             thetaspec_b[b] = asp[temp_ind] * del_omega_rad_b[b]
                             ;thetaspec_b[b] = asp[temp_ind] * finite(asp[temp_ind]);testing line
                          endfor

                       endif else begin
                          thetaspec_b = 0.
                          del_omega_rad_b = 0.
                       endelse
                       thetaspec_c = fltarr(nlarge_dtheta)
                       del_omega_rad_c = thetaspec_c

                       for c = 0,nlarge_dtheta-1 do begin
                          temp_ind = combins[large_dtheta_ind[c]]
                          th0_rad = (atheta[temp_ind] + dtheta[temp_ind]/2.)*!pi/180.
                          th1_rad = (atheta[temp_ind] - dtheta[temp_ind]/2.)*!pi/180.
                          if strmid(instrument,1,1) eq 's' then begin
                             w = maxphi_red[a] - minphi_red[a] ; fix in absence of sst phi-collection range
                          endif else begin
                             w = maxphi_red[a] - minphi_red[a] + $
                                 (maxphi_red[a] - minphi_red[a]) / (n_en_binsi - 1)
                          endelse
                          w_rad = w * !pi/180
                          del_omega_rad_c[c] = w_rad * (sin(th0_rad) - sin(th1_rad))
                          if ~ finite(asp[temp_ind]) then del_omega_rad_c[c] = 0.
                          thetaspec_c[c] = asp[temp_ind] * del_omega_rad_c[c]
                          ;thetaspec_c[c] = asp[temp_ind] * finite(asp[temp_ind]);testing line
                       endfor

                       del_omega_rad2 = total(del_omega_rad_b) + total(del_omega_rad_c)
                       thetaspec[i,a] = (total(thetaspec_b,/NAN) + total(thetaspec_c,/NAN)) / del_omega_rad2
                       ;thetaspec[i,a] = (total(thetaspec_b,/NAN) + total(thetaspec_c,/NAN));testing line
                    endfor ; loop over theta's

                 endif else begin
                    ;all bins have zero spectra data
                    if i gt 0 then begin
                       ntheta_bins = n_elements(thetaspec[i-1])
                       thetaspec[i] = fltarr(ntheta_bins)
                    endif else begin
                      thetaspec[i,*] = 0; the initial size of the thetaspec array
                      ntheta = n_elements(thetaspec[i,*])
                      atheta_red = fltarr(ntheta)
                      dtheta_red = fltarr(ntheta)
                    endelse
                 endelse
                 angspec = thetaspec
                 ; end compute theta angle spectra




                 endif else if doangle eq 'pa' then begin
                 ; compute pitch angle spectra
                 asp = nd eq 1 ? udat.data : total(udat.data * (bins ne 0),1) / total(bins ne 0,1) ; avg over all nrg bins
                 finite_ind = where(finite(asp),COMPLEMENT=infinite_ind, NCOMPLEMENT=ninfinite)
                 if ninfinite gt 0 then begin
                    asp[infinite_ind] = 0
                 endif
                 bad_ang = where(finite(total(udat.bins,1),/NAN),n); find sst "sunmasked" bins
                 if n gt 0 then asp[bad_ang]=!values.f_nan ; re-insert NaNs for sunmasked" bins

;                 ;test section
;                 asp = fltarr(88) + 1e5 ;test remove when done
;                 asp = fltarr(64) + 1e5 ;test remove when done
;                 ;aspi1=[0,4,8,9,16,17];,24,40,56,72]
;                 aspi1=[0,1,2,3,4,5,6,7]
;                 aspi2=[4,5,6,7]
;                 asp(0:3) = 1e4
;                 asp(16:19) = 1e6
;                 aspi3=[1,2,3,4,10,11,18,19,5,6,7,8,30,62,78,46,31,63,77,47,28,60,76,44]
;                 asp[aspi3] = 0

                 en_binsi = where(en_bins gt 0, n_en_binsi)
                 dphi = average(udat.dphi[en_binsi,*],1)
                 dtheta = average(udat.dtheta[en_binsi,*],1)
                 ;domega = average(udat.domega[en_binsi,*],1)
                 asp_on = intarr(n_elements(asp))
                 asp_on_ind = where(asp gt 0,n_asp_on_ind) ; index of angle bin where spectra has been calc'd
                 if n_asp_on_ind gt 0 then begin
                    an_bins_ind = where(an_bins eq 1)
                    asp_on(asp_on_ind) = 1
;                    aphi = average(udat.phi[en_binsi,*],1) ; avg phi of nrgies collected in each angle bin
                    aphi = average(udat.phi,1) ; avg phi of all available nrgies in each angle bin
;                    maxphi = max(udat.phi[en_binsi,*],dimension=1, min=minphi) ; min/max phi of energies collected in each angle bin
                    ; use this if you want the min/max phi of each bin regardless of nrgies picked
                    maxphi = max(udat.phi,dimension=1, min=minphi)
                    if array_equal(maxphi, aphi) then begin
                       maxphi = aphi + dphi/2
                       minphi = aphi - dphi/2
                    endif

                    ; convert any phi's gt 360
                    gt360_ind = where(aphi gt 360,gt360_count)
                    if gt360_count ne 0 then begin
                       aphi(gt360_ind) = aphi(gt360_ind) - 360
                       maxphi(gt360_ind) = maxphi(gt360_ind) - 360
                       minphi(gt360_ind) = minphi(gt360_ind) - 360
                    endif

                    atheta = average(udat.theta[en_binsi,*],1) ; avg theta of energies collected in each angle bin
                    aphi_on = aphi(an_bins_ind) ; array of aphi reduced to only bins that are on
                    maxphi_on = maxphi(an_bins_ind)
                    minphi_on = minphi(an_bins_ind)
                    atheta_on = atheta(an_bins_ind) ; array of atheta reduced to only bins that are on
                    dphi_on = average(udat.dphi(0,an_bins_ind),1)
                    dtheta_on = average(udat.dtheta(0,an_bins_ind),1)
                    domega_on = domega(an_bins_ind)
                    ; might have to test the fact that SORT might treat identical
                    ; elements differently on different OS's won't affect output
;                    phi_dat_ind = uniq(aphi_on, sort(aphi_on)); udat.phi bin #'s of unique and sorted phi's
;                    aphi_red = aphi_on(phi_dat_ind) ; sort and pick out unique avgphi's
;                    maxphi_red = maxphi_on(phi_dat_ind)
;                    minphi_red = minphi_on(phi_dat_ind)
;                    atheta_red = atheta_on(phi_dat_ind) ; sort and pick out unique avgtheta's
;                    dphi_red = dphi_on(phi_dat_ind)
;                    nphi = n_elements(aphi_red)
                    mag_vec = reform(magf[i,*])
                    p_angle = pangle(atheta_on, aphi_on, vec=mag_vec)
                    pa_sort_ind = sort(p_angle)
                    pa_red[i,*] = p_angle[pa_sort_ind]
                    npa = n_elements(p_angle)
                    paspec[i,*] = asp[pa_sort_ind]
                 endif else begin
                    ;all bins have zero spectra data
;                    if i gt 0 then begin
;                       npa_bins = n_elements(paspec[i-1])
;                       paspec[i] = fltarr(npa_bins)
;                    endif else begin
;                      paspec[i,*] = 0; the initial size of the paspec array
;                      npa = n_elements(paspec[i,*])
;                      apa_red = fltarr(npa)
;                      dpa_red = fltarr(npa)
;                    endelse
                 endelse
                 angspec = paspec
                 ; see pangle.pro

                 ; end compute pitch angle spectra





                 endif else if doangle eq 'gphi' then begin
                 ; compute pitch angle spectra
                 dprint, 'Gyrophase spectrograms not yet implemented.'
                 doangle = 'none'
                 continue
                 ; end compute pitch angle spectra

                 endif else if doangle eq 'none' then begin
                    angspec = 0
                 continue

                 endif else begin
                 dprint, 'ERROR: ', doangle, ' is not a valid spectrogram
                 doangle = 'none'
                 endelse


              endif
;              if  nmoments ne 0 then begin
;;                  dat.dead = 0
;                  if keyword_set(magf) then  dat.magf=magf[i,*]
;                  if keyword_set(scpot) then dat.sc_pot=scpot[i]
;                  moms[i] = moments_3d( dat , domega=domega )
;              endif
              dprint,dwait=10.,format,i,'/',ns;,'    ',time_string(dat.time)   ; update every 10 seconds
           endfor

           finite_ind = where(finite(angspec),COMPLEMENT=infinite_ind, NCOMPLEMENT=ninfinite)
           if ninfinite gt 0 then begin
              angspec[infinite_ind] = 0
           endif

           ;test section
           ;angspec[*,10]=0
           ;angspec[1,3]=0.1
           ;end test section

           if not keyword_set(no_tplot) then begin
              prefix = thx+'_'+instrument+'_'
              suffix = '_'+strlowcase(units)+tplotsuffix
              if keyword_set(units) then begin
                 ; beg create tplot vars for energy spectra
                 if keyword_set(doenergy) then begin
                   tname = prefix+'en'+suffix
                   spec_ind = where(total(spec,1,/NAN) gt 0,n)
                   if n eq 1 then begin
                      store_data,tname, data={x:time, y:spec[*,spec_ind], v:energy[*,spec_ind]},$
                                 dlim={spec:1,zlog:1,ylog:1}
                   endif else begin
                      store_data,tname, data={x:time, y:spec[*,spec_ind[0]:spec_ind[n-1]], $
                      v:energy[*,spec_ind[0]:spec_ind[n-1]]}, dlim={spec:1,zlog:1,ylog:1}
                   endelse
                 endif ; end create tplot vars for energy spectra

                 ; beg create tplot vars for phi angle spectra
                 if doangle eq 'phi' then begin
                   an_tname = prefix + 'an_eflux_' + doangle + tplotsuffix
                   ; beg re-sort of phi's so they go from 0-360
                   ps_size = n_elements(angspec[0,*])
                   if ps_size gt nphi then begin
                      angspec = angspec[*,0:nphi-1]
                      ps_size = n_elements(angspec[0,*])
                   endif

                   if keyword_set(start_angle) then begin
                      if start_angle lt 0 then begin
                         aphi_red = aphi_red - 360
                         st_ind = min(where(start_angle lt aphi_red))
                         aphi_red = shift(aphi_red, -st_ind)
                         too_low_ind = where(start_angle gt aphi_red)
                         aphi_red[too_low_ind] = aphi_red[too_low_ind] + 360
                      endif else begin
                         st_ind = min(where(start_angle lt aphi_red))
                         aphi_red = shift(aphi_red, -st_ind)
                         too_low_ind = where(start_angle gt aphi_red,n)
                         if n gt 0 then aphi_red[too_low_ind] = aphi_red[too_low_ind] + 360
                         ;aphi_red[too_low_ind] = aphi_red[too_low_ind] + 360
                      endelse
                   endif else st_ind = 0

                   if nd gt 1 then begin
                      if size(/n_dimensions,angspec) eq 1 then begin
                         angspec = reform(angspec,n_elements(angspec),1)
                      endif else begin
                         angspec = shift(angspec, 0, -st_ind)
                      endelse
                   endif

                   nwrap = 0
                   ; begin append wrapped phi bins
                   if wrapphi gt 0 then begin
                      wrap_ind = where((aphi_red gt start_angle) AND $
                                       (aphi_red lt (start_angle + wrapphi)),nwrap)
                      wrap_angspec = replicate(0.,ns, ps_size+nwrap)
                      wrap_angspec[*,0:nphi-1] = angspec
                      wrap_angspec[*,nphi:nphi + nwrap-1] = angspec[*,0:nwrap-1]

                      wrap_ps_size = n_elements(wrap_angspec[0,*])
                      wrap_aphi_red = replicate(0.,wrap_ps_size)
                      wrap_aphi_red[0:nphi-1] = aphi_red
                      wrap_aphi_red[nphi:nphi + nwrap-1] = aphi_red[0:nwrap-1] + 360

                      bnd_angspec = replicate(0.,ns,wrap_ps_size+2)
                      bnd_angspec[*,1:wrap_ps_size] = wrap_angspec
                      bnd_angspec[*,0] = angspec[*,ps_size-1]; might have to make this zeros or NaNs
                      bnd_angspec[*,wrap_ps_size+1] = angspec[*,nwrap] ; what if angspec[*,nwrap] doesn't exist?

                      bnd_aphi_red = replicate(0.,wrap_ps_size+2)
                      bnd_aphi_red[1:wrap_ps_size] = wrap_aphi_red
                      bnd_aphi_red[0] = aphi_red[ps_size-1] - 360
                      bnd_aphi_red[wrap_ps_size+1] = aphi_red[nwrap] + 360
                   endif else begin ; begin append wrapped phi bins if necceary
                      if (phi[1] - phi[0]) lt 360 then begin
                         bnd_angspec = angspec
                         bnd_aphi_red = aphi_red
                         p_start_angle = aphi_red[0]
                         p_end_angle = aphi_red[nphi-1]
                         if n_elements(angspec[0,*]) eq 1 then begin
                            p_start_angle = minphi_red
                            p_end_angle = maxphi_red
                            bnd_angspec = replicate(0.,ns,ps_size+nwrap+2)
                            bnd_angspec[*,1:nphi] = angspec
                            bnd_angspec[*,0] = angspec
                            bnd_angspec[*,nphi+1] = angspec
                            ; end get y boundaries correct
                            ; look for phi angles gt 360 and wrap around to 0
                            bnd_aphi_red = replicate(0.,ps_size+2)
                            bnd_aphi_red[1:nphi] = aphi_red
                            bnd_aphi_red[0] = minphi_red
                            bnd_aphi_red[nphi+1] = maxphi_red
                         endif
                      endif else begin
                         ; begin get y boundaries correct
                         bnd_angspec = replicate(0.,ns,ps_size+nwrap+2)
                         bnd_angspec[*,1:nphi] = angspec
                         bnd_angspec[*,0] = angspec[*,nphi-1]
                         bnd_angspec[*,nphi+1] = angspec[*,0]
                         ; end get y boundaries correct
                         ; look for phi angles gt 360 and wrap around to 0
                         bnd_aphi_red = replicate(0.,ps_size+2)
                         bnd_aphi_red[1:nphi] = aphi_red
                         bnd_aphi_red[0] = aphi_red[nphi-1]-360
                         bnd_aphi_red[nphi+1] = aphi_red[0]+360
                      endelse
                   endelse
                   store_data,an_tname, data= {x:time, y:bnd_angspec ,v:bnd_aphi_red},dlim={spec:1,zlog:1,ylog:0}
                   ylim,'th*an_eflux*',p_start_angle, p_end_angle,log=0
                 endif ; end create tplot vars for phi angle spectra



                 ; beg create tplot vars for theta angle spectra
                 if doangle eq 'theta' then begin
                   an_tname = prefix + 'an_eflux_' + doangle + tplotsuffix
                   ; beg re-sort of theta's so they go from -90 to 90
                   ps_size = n_elements(angspec[0,*])
                   if ps_size gt ntheta then begin
                      angspec = angspec[*,0:ntheta-1]
                      ps_size = n_elements(angspec[0,*])
                   endif

                   if 0 then begin ;keyword_set(start_angle) then begin
                      if start_angle lt 0 then begin
                         aphi_red = aphi_red - 360
                         st_ind = min(where(start_angle lt aphi_red))
                         aphi_red = shift(aphi_red, -st_ind)
                         too_low_ind = where(start_angle gt aphi_red)
                         aphi_red[too_low_ind] = aphi_red[too_low_ind] + 360
                      endif else begin
                         st_ind = min(where(start_angle lt aphi_red))
                         aphi_red = shift(aphi_red, -st_ind)
                         too_low_ind = where(start_angle gt aphi_red,n)
                         if n gt 0 then aphi_red[too_low_ind] = aphi_red[too_low_ind] + 360
                         ;aphi_red[too_low_ind] = aphi_red[too_low_ind] + 360
                      endelse
                   endif else st_ind = 0

                   if nd gt 1 then begin
                      if size(/n_dimensions,angspec) eq 1 then begin
                         angspec = reform(angspec,n_elements(angspec),1)
                      endif else begin
                         angspec = shift(angspec, 0, -st_ind)
                      endelse
                   endif

                   nwrap = 0
                   ; begin append wrapped theta bins
;                   if wrapphi gt 0 then begin
;                      wrap_ind = where((aphi_red gt start_angle) AND $
;                                       (aphi_red lt (start_angle + wrapphi)),nwrap)
;                      wrap_angspec = replicate(0.,ns, ps_size+nwrap)
;                      wrap_angspec[*,0:nphi-1] = angspec
;                      wrap_angspec[*,nphi:nphi + nwrap-1] = angspec[*,0:nwrap-1]
;
;                      wrap_ps_size = n_elements(wrap_angspec[0,*])
;                      wrap_aphi_red = replicate(0.,wrap_ps_size)
;                      wrap_aphi_red[0:nphi-1] = aphi_red
;                      wrap_aphi_red[nphi:nphi + nwrap-1] = aphi_red[0:nwrap-1] + 360
;
;                      bnd_angspec = replicate(0.,ns,wrap_ps_size+2)
;                      bnd_angspec[*,1:wrap_ps_size] = wrap_angspec
;                      bnd_angspec[*,0] = angspec[*,ps_size-1]; might have to make this zeros or NaNs
;                      bnd_angspec[*,wrap_ps_size+1] = angspec[*,nwrap] ; what if angspec[*,nwrap] doesn't exist?
;
;                      bnd_aphi_red = replicate(0.,wrap_ps_size+2)
;                      bnd_aphi_red[1:wrap_ps_size] = wrap_aphi_red
;                      bnd_aphi_red[0] = aphi_red[ps_size-1] - 360
;                      bnd_aphi_red[wrap_ps_size+1] = aphi_red[nwrap] + 360
;                   endif else begin ; begin append wrapped theta bins if necceary
                      if (theta[1] - theta[0]) lt 180 then begin
                         bnd_angspec = angspec
                         bnd_atheta_red = atheta_red
                         th_start_angle = atheta_red[0]
                         th_end_angle = atheta_red[ntheta-1]
                         if n_elements(angspec[0,*]) eq 1 then begin
                            th_start_angle = mintheta_red
                            th_end_angle = maxtheta_red
                            bnd_angspec = replicate(0.,ns,ps_size+nwrap+2)
                            bnd_angspec[*,1:ntheta] = angspec
                            bnd_angspec[*,0] = angspec
                            bnd_angspec[*,ntheta+1] = angspec
                            ; end get y boundaries correct
                            ; look for phi angles gt 90 and wrap around to -90
                            bnd_atheta_red = replicate(0.,ps_size+2)
                            bnd_atheta_red[1:ntheta] = atheta_red
                            bnd_atheta_red[0] = mintheta_red
                            bnd_atheta_red[ntheta+1] = maxtheta_red
                         endif
                      endif else begin
                         ; begin get y boundaries correct
                         bnd_angspec = replicate(0.,ns,ps_size+nwrap+2)
                         bnd_angspec[*,1:ntheta] = angspec
                         bnd_angspec[*,0] = angspec[*,ntheta-1]
                         bnd_angspec[*,ntheta+1] = angspec[*,0]
                         ; end get y boundaries correct
                         ; look for theta angles gt 90 and wrap around to -90
                         bnd_atheta_red = replicate(0.,ps_size+2)
                         bnd_atheta_red[1:ntheta] = atheta_red
                         bnd_atheta_red[0] = atheta_red[ntheta-1]-180
                         bnd_atheta_red[ntheta+1] = atheta_red[0]+180
                      endelse
;                   endelse
                   store_data,an_tname, data= {x:time, y:bnd_angspec ,v:bnd_atheta_red},dlim={spec:1,zlog:1,ylog:0}
                   if theta1 gt th_start_angle then th_start_angle = theta1
                   if theta2 lt th_end_angle then th_end_angle = theta2
                   ylim,'th*an_eflux*',th_start_angle, th_end_angle,log=0
                 endif ; end create tplot vars for theta angle spectra




                 ; beg create tplot vars for pitch angle angle spectra
                 if doangle eq 'pa' then begin
                   an_tname = prefix + 'an_eflux_' + doangle + tplotsuffix
                   ; beg re-sort of pa's so they go from -90 to 90
                   ps_size = n_elements(angspec[0,*])
                   if ps_size gt npa then begin
                      angspec = angspec[*,0:npa-1]
                      ps_size = n_elements(angspec[0,*])
                   endif

                   if 0 then begin ;keyword_set(start_angle) then begin
                      if start_angle lt 0 then begin
                         aphi_red = aphi_red - 360
                         st_ind = min(where(start_angle lt aphi_red))
                         aphi_red = shift(aphi_red, -st_ind)
                         too_low_ind = where(start_angle gt aphi_red)
                         aphi_red[too_low_ind] = aphi_red[too_low_ind] + 360
                      endif else begin
                         st_ind = min(where(start_angle lt aphi_red))
                         aphi_red = shift(aphi_red, -st_ind)
                         too_low_ind = where(start_angle gt aphi_red,n)
                         if n gt 0 then aphi_red[too_low_ind] = aphi_red[too_low_ind] + 360
                         ;aphi_red[too_low_ind] = aphi_red[too_low_ind] + 360
                      endelse
                   endif else st_ind = 0

                   if nd gt 1 then begin
                      if size(/n_dimensions,angspec) eq 1 then begin
                         angspec = reform(angspec,n_elements(angspec),1)
                      endif else begin
                         angspec = shift(angspec, 0, -st_ind)
                      endelse
                   endif

                   nwrap = 0
                   ; begin append wrapped theta bins
;                   if wrapphi gt 0 then begin
;                      wrap_ind = where((aphi_red gt start_angle) AND $
;                                       (aphi_red lt (start_angle + wrapphi)),nwrap)
;                      wrap_angspec = replicate(0.,ns, ps_size+nwrap)
;                      wrap_angspec[*,0:nphi-1] = angspec
;                      wrap_angspec[*,nphi:nphi + nwrap-1] = angspec[*,0:nwrap-1]
;
;                      wrap_ps_size = n_elements(wrap_angspec[0,*])
;                      wrap_aphi_red = replicate(0.,wrap_ps_size)
;                      wrap_aphi_red[0:nphi-1] = aphi_red
;                      wrap_aphi_red[nphi:nphi + nwrap-1] = aphi_red[0:nwrap-1] + 360
;
;                      bnd_angspec = replicate(0.,ns,wrap_ps_size+2)
;                      bnd_angspec[*,1:wrap_ps_size] = wrap_angspec
;                      bnd_angspec[*,0] = angspec[*,ps_size-1]; might have to make this zeros or NaNs
;                      bnd_angspec[*,wrap_ps_size+1] = angspec[*,nwrap] ; what if angspec[*,nwrap] doesn't exist?
;
;                      bnd_aphi_red = replicate(0.,wrap_ps_size+2)
;                      bnd_aphi_red[1:wrap_ps_size] = wrap_aphi_red
;                      bnd_aphi_red[0] = aphi_red[ps_size-1] - 360
;                      bnd_aphi_red[wrap_ps_size+1] = aphi_red[nwrap] + 360
;                   endif else begin ; begin append wrapped theta bins if necceary
                      if (pitch[1] - pitch[0]) le 180 then begin
                         bnd_angspec = angspec
                         bnd_pa_red = pa_red
                         th_start_angle = min(pa_red)
                         th_end_angle = max(pa_red)
                         if n_elements(angspec[0,*]) eq 1 then begin
                            th_start_angle = pa_red - 0.5 ; arbitrary bouns
                            th_end_angle = pa_red + 0.5 ; arbitrary bounds
                            bnd_angspec = replicate(0.,ns,ps_size+nwrap+2)
                            bnd_angspec[*,1:npa] = angspec
                            bnd_angspec[*,0] = angspec
                            bnd_angspec[*,npa+1] = angspec
                            ; end get y boundaries correct
                            ; look for phi angles gt 90 and wrap around to -90
                            bnd_pa_red = replicate(0.,ps_size+2)
                            bnd_pa_red[1:npa] = pa_red
                            bnd_pa_red[0] = pa_red - 0.5 ; arbitrary bouns
                            bnd_pa_red[npa+1] = pa_red + 0.5 ; arbitrary bounds
                         endif
                      endif else begin
                         ; begin get y boundaries correct
                         bnd_angspec = replicate(0.,ns,ps_size+nwrap+2)
                         bnd_angspec[*,1:npa] = angspec
                         bnd_angspec[*,0] = angspec[*,npa-1]
                         bnd_angspec[*,npa+1] = angspec[*,0]
                         ; end get y boundaries correct
                         ; look for theta angles gt 90 and wrap around to -90
                         bnd_pa_red = replicate(0.,ps_size+2)
                         bnd_pa_red[1:npa] = pa_red
                         bnd_pa_red[0] = pa_red[npa-1]-180
                         bnd_pa_red[npa+1] = pa_red[0]+180
                      endelse
;                   endelse
                   store_data,an_tname, data= {x:time, y:bnd_angspec ,v:bnd_pa_red},dlim={spec:1,zlog:1,ylog:0}
                   if pitch1 gt th_start_angle then th_start_angle = pitch1
                   if pitch2 lt th_end_angle then th_end_angle = pitch2
                   ylim,'th*an_eflux*',th_start_angle, th_end_angle,log=0
                 endif ; end create tplot vars for theta angle spectra



              endif
;              for i = 0, nmoments-1 do begin
;                  momname = moments_a[i]
;                  value = reform(transpose( struct_value(moms,momname) ) )
;                  tname = prefix + momname
;                  append_array,tplotnames,tname
;;                  printdat,value,varname= comps[i]
;                  store_data,tname,data= { x: moms.time,  y: value }
;                  if size(/n_dimen,value) gt 1 then options,tname,colors='bgr',/def
;              endfor
           endif
        endif
    endfor
endfor
dprint,dlevel=3,verbose=verbose,'Run Time: ',systime(1)-start,' seconds

tn=tplotnames
options,strfilter(tn,'*_density'),/def ,yrange=[.01,200.],/ystyle,/ylog,ysubtitle='!c[1/cc]'
options,strfilter(tn,'*_velocity'),/def ,yrange=[-800,800.],/ystyle,ysubtitle='!c[km/s]'
options,strfilter(tn,'*_flux'),/def ,yrange=[-1e8,1e8],/ystyle,ysubtitle='!c[#/s/cm2 ??]'
options,strfilter(tn,'*t3'),/def ,yrange=[1,10000.],/ystyle,/ylog,ysubtitle='!c[eV]'

end