;+
;PROCEDURE: thm_part_getspec
;PURPOSE:
;	Generates tplot variables containing energy-time angular particle spectra
;	for any angle range and/or any energy range.
;
;;keywords:
;  probe = Probe name. The default is 'all', i.e., load all available probes.
;          This can be an array of strings, e.g., ['a', 'b'] or a
;          single string delimited by spaces, e.g., 'a b'
;  trange= (Optional) Time range of interest  (2 element array), if
;          this is not set, the default is to prompt the user. Note
;          that if the input time range is not a full day, a full
;          day's data is loaded.
;  phi   = Angle range of interest (2 element array) in degrees relative to
;          probe-sun direction in the probe's spin plane. Specify angles in
;          ascending order (e.g. [270, 450]) to specify the 'daylight'
;          hemisphere in DSL coordinates. Default is all 360 degrees (e.g.
;          [0, 360]). If the phi range is greater than 360 degrees, then the phi
;          bins at the beginning of the phi range are added and wrapped around
;          the end of phi range. For example, if phi=[0,420], the phi bins
;          corresponding to 0-60 degrees are appended to the top of the plot.
;  theta = Angle range of interest (2 element array) in degrees relative to
;          spin plane, e.g. [-90, 0] or [-45, 45] in the probe's spin plane.
;          Specify in acending order. Default is all (e.g. [-90, 90]).
;  pitch = Angle range of interest (2 element array) in degrees relative to
;          the magnetic field. Default is all (e.g. [0, 180]).
;  erange= Energy range (in eV) of interest (2 element array). Default is all.
;  data_type = The type of data to be loaded. eESA reduced mode now working.
;  start_angle = angle at which to begin plotting the data the bottom of the
;          plot along the y-axis. At this time this works for phi only. If not
;		   set, keyword defaults to first element of phi input.
;  suffix= suffix to add to tplot variable names.
;  /AUTOPLOT: Set to activiate a simple plotting routine to display tplot
;          variables created by this routine.
;  /ENERGY: Set to output tplot variables containing the energy spectrum of the
;          energy/angle ranges input above
;  angle = Type of angular spectrum tplot variable created and plotted using the
;          energy/angle ranges input above e.g. 'phi'. Default is 'phi'. The
;          type of angular spectra is appended to its corresponding tplot
;          variable name. All possible options are:
;          phi   = phi angular spectrum
;          theta = theta angular spectrum
;          pa    = pitch angular spectrum
;          gphi  = gyrophase angular spectrum (not yet enabled)
;          NOTE: If neither of ENERGY and ANGLE keywords are specified, then
;                then both are turned on with ANGLE defaulting to 'phi'.
;  /NOMASK: Set to turn off default masking of sun pulse in SST data
;  /GET_SUPPORT_DATA: load support_data variables as well as data variables
;          into tplot variables.
;
;EXAMPLE: thm_part_getspec, probe='d', trange=['07-06-17','07-06-19']
;
;SEE ALSO:
;	THM_CRIB_PART_GETSPEC, THM_PART_MOMENTS2, THM_PART_GETANBINS, THM_LOAD_SST,
;   THM_LOAD_ESA_PKT
;
;CREATED BY:	Bryan Kerr
;HISTORY:
; v0.1	11/20/07: Initial release.
; v0.2	11/28/07: Added ability to handle eESA and *SST data types
; v0.3  12/04/07: Added st_phi, tplotsuffix and autoplot keywords
; v0.4  12/11/07: Added ability to plot energy spectra,angle/energy keywords and
;                 angle/energy tplot variable setup routines. Also added wrapphi
;                 ability.
; v0.4.1 12/13/07:Set ANGLE and ENERGY keywords to both turn on if neither is
;                 specified. Moved start/end_angle module to THM_PART_MOMENTS2.
; v0.5  01/04/08: Changed tplotsuffix to suffix. Added NOMASK keyword. Reduced
;                 banding of masked SST data. ANGLE keyword now specifies type
;                 of angular spectrum.
; v0.6  01/09/08: Added reduced mode (peir) capability.
; v0.6.3 01/15/08: Implemented all reduced modes.
; v0.7  01/31/08: Added theta spectragram capability.
; v0.8  02/04/08: Alpha release of pitch angle spectragram capability.
; v0.8.01 02/06/08: Fixed bug that bombs thm_part_getspec if angle keyword not
;                   set.
; v0.8.1 02/20/08: Added /GET_SUPPORT_DATA keyword. Modified code to better
;                  handle 6-angle data.
; v0.8.11 02/20/08: Tplot title only created when /AUTOPLOT is specified.
;
; NOTES: There's a bug that causes the code to bomb when a phi range is
;        specified for theta spectragrams.
;
;VERSION: 0.8.11
;  $LastChangedBy: bckerr $
;  $LastChangedDate: 2008-02-20 14:41:00 -0800 (Wed, 20 Feb 2008) $
;  $LastChangedRevision: 2388 $
;  $URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/trunk/idl/themis/spacecraft/particles/thm_part_getspec.pro $
;-

pro thm_part_getspec, probe=probe, trange=trange, phi=phi, theta=theta, $
                      pitch=pitch, erange=erange, data_type=data_type, $
                      start_angle=start_angle, suffix=suffix, $
                      autoplot=autoplot, angle=angle, energy=energy, $
                      nomask=nomask, get_support_data=get_support_data

; Save for later
;defprobes = '?'
;definstruments='p??f'

deftheta = [-90, 90]
defphi = [0, 360]
defpitch = [0, 180]
deferange = [0, 1e7]
defdata_type = 'peif'
def_angle = 'phi'
defstart_angle = 0
wrapphi = 0

if keyword_set(theta) then begin
   if theta[1] lt theta[0] OR (theta[0] lt -90 OR theta[1] gt 90) then begin
      print,''
      dprint, 'Error with theta input. Does not satisfy: -90 < theta[0] < theta[1] < 90'
      return
   endif
endif else theta=deftheta

if keyword_set(phi) then begin
   if phi[0] gt phi[1] then begin
      print,''
      dprint, 'Error with phi input. Phi angles must be specified in ascending order'
      return
   endif
   if (phi[1] - phi[0]) gt 360 then begin
      wrapphi = phi[1] - phi[0] - 360
      phi[1] = phi[0] + 360
   endif
endif else phi=defphi

if ~ keyword_set(angle) AND ~ keyword_set(energy) then begin
   angle = def_angle
   dprint, 'setting angle data_type: ', angle
   energy=1
endif else if ~ keyword_set(angle) then angle = 0

if keyword_set(pitch) eq 0 then pitch=defpitch
if keyword_set(erange) eq 0 then erange=deferange
if keyword_set(data_type) eq 0 then begin
   data_type = defdata_type
   print,''
   dprint, 'setting default data_type: ', data_type
   print,''
endif

theta1 = min(theta)
theta2 = max(theta)
phi1 = phi[0]
phi2 = phi[1]

tr = time_double(trange)
ndays = (tr[1] - tr[0]) / 86400
timespan,tr[0],ndays


; Generate angle/energy spectrum by calling thm_part_moments
datatype = data_type

sst_ind = where(strmid(data_type,1,1) eq 's', sst_count)
if sst_count ne 0 then begin
   sst_type = data_type[sst_ind]
   thm_load_sst, probe=probe, trange=trange, datatype=sst_type, $
                get_support_data=get_support_data
endif

esa_ind = where(strmid(data_type,1,1) eq 'e', esa_count)
if esa_count ne 0 then begin
   esa_type = data_type[esa_ind]
   thm_load_esa_pkt, probe=probe, trange=trange, datatype=esa_type, $
                    get_support_data=get_support_data
endif

if angle eq 'pa' then begin
   coord = 'dsl
   mag_suffix = '_fgs_'+coord+'_'+angle
   thm_load_fgm, probe=probe, datatype='fgs', suffix='_'+angle, coord='dsl', $
                get_support_data=get_support_data
endif

thm_part_moments2, probe=probe, instruments_types=data_type, trange=trange, $
                  tplotnames=tn, theta=theta, phi=phi, pitch=pitch, $
                  erange=erange, tplotsuffix=suffix, start_angle=start_angle, $
                  doangle=angle, doenergy=energy, wrapphi=wrapphi, $
                  nomask=nomask, mag_suffix=mag_suffix


; Setup angle/energy spectrum tplot variables calculated by thm_part_moments2

if keyword_set(angle) then begin
   stheta = strcompress(string(theta))
   sphi = strcompress(string(phi))
   serang = strcompress(string(erange))
   title = strjoin(['theta=', stheta,', phi=',sphi,', erange=', $
                    serang])
   tplot_options,'title',''
   options,'th*an_eflux*','y_no_interp',1
   options,'th*an_eflux*','x_no_interp',1
   options,'th*an_eflux*','ztitle','Eflux [eV/cm2/s/ster/eV]'
   options,'th*an_eflux*',minzlog=1
   zlim,'th*an_eflux*',1,1,1
;   ylim,'th*an_eflux*',p_start_angle, p_end_angle,log=0
   ;tdegap, 'th*an_eflux', overwrite=1
endif

if keyword_set(energy) then begin
   stheta = strcompress(string(theta))
   sphi = strcompress(string(phi))
   serang = strcompress(string(erange))
   title = strjoin(['theta=', stheta,', phi=',sphi,', erange=', $
                    serang])
   tplot_options,'title',''
   options,'th*en_eflux*','y_no_interp',1
   options,'th*en_eflux*','x_no_interp',1
   options,'th*en_eflux*','ztitle','Eflux [eV/cm2/s/ster/eV]'
   options,'th*en_eflux*',minzlog=1
   zlim,'th*en_eflux*',1,1,1
   options,'th*en_eflux*',ystyle=1
   ;tdegap, 'th*an_eflux', overwrite=1
endif

; Autoplot the angle/energy spectra
if keyword_set(autoplot) && keyword_set(angle) then begin
   tplot_options,'title',title
   tplot,'th*_an_eflux*'
endif
if keyword_set(autoplot) && keyword_set(energy) then begin
   if keyword_set(angle) then begin
      tplot_options,'title',title
      tplot, 'th*eflux*'
   endif else begin
      tplot_options,'title',title
      tplot,'th*_en_eflux*'
   endelse
endif
end